###############################################################################
#
# Developed with love
#
#  (c) 2021-2021 Copyright: Sebastian Schwarz 
#
#  This script is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  any later version.
#
#  The GNU General Public License can be found at
#  http://www.gnu.org/copyleft/gpl.html.
#  A copy is found in the textfile GPL.txt and important notices to the license
#  from the author is found in LICENSE.txt distributed with these scripts.
#
#  This script is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#
# $Id$
#
###############################################################################

package FHEM::Hailo::Libero;

use strict;
use warnings;
#use POSIX;

use Data::Dumper;


use FHEM::Meta;
use GPUtils qw(GP_Import);
use HttpUtils;
use FHEM::Core::Authentication::Passwords qw(:ALL);


my $missingModul = ''; 

## Import der FHEM Funktionen
#-- Run before package compilation
BEGIN {

    # Import from main context
    GP_Import(
        qw(readingsSingleUpdate
          readingsBulkUpdate
          readingsBulkUpdateIfChanged
          readingsBeginUpdate
          readingsEndUpdate
          Log3
          CommandAttr
          AttrVal
          ReadingsVal
          CommandDefMod
          modules
          setKeyValue
          getKeyValue
          getUniqueId
          RemoveInternalTimer
          InternalTimer
          defs
          init_done
          IsDisabled
          deviceEvents
          HttpUtils_NonblockingGet
          gettimeofday
          Dispatch)
    );
}

sub Define {
    my $hash = shift // return;
    my $aArg = shift // return;

    return $@ unless ( FHEM::Meta::SetInternals($hash) );
    use version 0.60; our $VERSION = FHEM::Meta::Get( $hash, 'version' );

    return q{too few parameters: define <NAME> HailoLibero}
      if ( scalar( @{$aArg} ) != 2 );
    
    my $name = shift @$aArg;
    $hash->{URL} =
      AttrVal( $name, 'LiberoIP', 'http://192.168.0.1:81');

    $hash->{NOTIFYDEV} = "global,$name";
    $hash->{VERSION}    = version->parse($VERSION)->normal;
   
    CommandAttr( undef, $name . ' room Hailo' )
      if ( AttrVal( $name, 'room', 'none' ) eq 'none' );

    readingsSingleUpdate( $hash, 'state', 'initialized', 1 );
    readingsSingleUpdate( $hash, 'cookie', 'none', 1 );


    Log3($name, 3, qq{HailoLibero ($name) - defined HailoLibero});

    ### create password object to handle pass keystore
    $hash->{helper}->{passObj} = FHEM::Core::Authentication::Passwords->new($hash->{TYPE});
    
    return;
}

sub Undef {
    my $hash = shift;
    my $name = shift;

    $hash->{helper}->{passObj}->setDeletePassword($name);
    
    return;
}

sub Attr {
    my ( $cmd, $name, $attrName, $attrVal ) = @_;
    my $hash = $defs{$name};

    if ( $attrName eq 'disable' ) {
        if ( $cmd eq 'set' && $attrVal eq '1' ) {
            RemoveInternalTimer($hash);
            readingsSingleUpdate( $hash, 'state', 'inactive', 1 );
            Log3($name, 3, qq{HailoLibero ($name) - disabled});
        }
        elsif ( $cmd eq 'del' ) {
            readingsSingleUpdate( $hash, 'state', 'active', 1 );
            Log3($name, 3, qq{HailoLibero ($name) - enabled});
        }
    }
    elsif ( $attrName eq 'LiberoIP') {
      if ( $cmd eq 'set' ) {
        $hash->{URL} = "http://$attrVal:81";
        Log3 $name, 3,
        "HailoLibero ($name) - set LiberoIP to: $attrVal";
      } elsif ( $cmd eq 'del' ) {
        $hash->{URL} = 'http://192.168.0.1:81';
      }
    }

    return;
}


sub Notify {
    my $hash = shift // return;
    my $dev  = shift // return;

    my $name = $hash->{NAME};
    return if ( IsDisabled($name) );

    my $devname = $dev->{NAME};
    my $devtype = $dev->{TYPE};
    my $events  = deviceEvents( $dev, 1 );

    return if ( !$events );

    getCookie($hash)
      if (
        (
            $devtype eq 'Global'
            && (
                grep /^INITIALIZED$/,
                @{$events} or grep /^REREADCFG$/,
                @{$events} or grep /^DEFINED.$name$/,
                @{$events} or grep /^MODIFIED.$name$/,
                @{$events}
            )
        )

        || (
            $devtype eq 'HailoLibero'
            && (
                grep /^LiberoIP.+/,
                @{$events}
            )
        )
      );


    if (
        $devtype eq 'HailoLibero'
        && (
            grep /^cookie:.*$/,
            @{$events}
        )
        && (
          ReadingsVal( $name, 'cookie', 'none' ) ne 'none'
        )
      )
    {
        getSettings($hash);
        #InternalTimer( gettimeofday() + $hash->{INTERVAL},
         #   "FHEM::HailoLibero::getSettings", $hash );
        Log3 $name, 4,
"HailoLibero ($name) - set internal timer function for recall getSettings sub";
    }

    return;
}

sub Set {
    my $hash = shift // return;
    my $aArg = shift // return;

    my $name = shift @$aArg // return;
    my $cmd  = shift @$aArg // return qq{"set $name" needs at least one argument};
         
    if ( lc $cmd eq 'password' ) {
        return qq{error set libero ip first}
         if ( AttrVal( $name, 'LiberoIP', 'none' ) eq 'none' );
      
        my ($passResp,$passErr);
        ($passResp,$passErr) = $hash->{helper}->{passObj}->setStorePassword($name,$aArg->[0]);
        
        return qq{error while saving the password - $passErr}
          if ( !defined($passResp)
           and defined($passErr) );

        return q{password successfully saved}
          if ( defined($passResp)
           and !defined($passErr) );
        getCookie($hash);
    } 
    elsif ( lc $cmd =~ /open|led_brightness|eject_power|eject_delay|detection_area|detection_sensitivity/ ) {
        return "please set Attribut LiberoIP first"
          if ( AttrVal( $name, 'LiberoIP', 'none' ) eq 'none' );
        
        getCookie($hash)
          if  (ReadingsVal( $name, 'cookie', 'none' ) eq 'none');
        
        # cmd = setter,  $aArg->[0] = value
        if (lc $cmd ne 'open') {
          my $payload = '';
          $payload = 'led='.$aArg->[0] if (lc $cmd eq 'led_brightness');
          $payload = 'pwr='.$aArg->[0]  if (lc $cmd eq 'eject_power');
          $payload = 'dist='.$aArg->[0]  if (lc $cmd eq 'detection_area');
          $payload = 'delay='.$aArg->[0]  if (lc $cmd eq 'detection_sensitivity');
          $payload = 'pdelay='.$aArg->[0]  if (lc $cmd eq 'eject_delay');
          Log3 $name, 3, "setter write $payload to libero";
          Write($hash, $payload);

          #getSettings($hash);
        } 

        # open door
        Write($hash,'open')
          if (lc $cmd eq 'open');

    } 
    else {
        my $list = 'password ';
        $list .= 'open:noArg led_brightness:slider,1,1,10 eject_power:slider,1,1,10 eject_delay:slider,0,1,5 detection_area:slider,31,1,100 detection_sensitivity:slider,1,1,50'
          if ( defined(ReadPassword( $hash, $name ))
            && AttrVal( $name, 'LiberoIP', 'none' ) ne 'none'
           );

        return "Unknown argument $cmd, choose one of $list";
    }

    return;
}

sub Delete {
    my $hash = shift;
    my $name = shift;

    $hash->{helper}->{passObj}->setDeletePassword($name);
    return;
}

sub getCookie {
    my $hash = shift;
    my $name = $hash->{NAME};
    Log3 $name, 3, "HailoLibero ($name) - catch cookies";

    Write (
      $hash,
      'pin='
      .ReadPassword($hash, $name)
      .'&submit='
    ) if ( defined(ReadPassword( $hash, $name ))
            && AttrVal( $name, 'LiberoIP', 'none' ) ne 'none'
           );
}

sub setSettings {
  my $hash = shift;
  my $name = $hash->{NAME};

  Write( $hash, 'settings', undef, undef );

}

sub getSettings {
  my $hash = shift;
  my $name = $hash->{NAME};
  
  Write( $hash, undef, undef, undef );
}

sub Write {
    my ( $hash, $payload ) = @_;
    my $name = $hash->{NAME};

    my ( $header, $uri, $method );

    ( $payload, $header, $uri, $method ) =
      createHttpValueStrings( $hash, $payload );
      
    HttpUtils_NonblockingGet(
        {
            url       => $hash->{URL} . $uri,
            timeout   => 15,
            hash      => $hash, 
            data      => $payload,
            method    => $method,
            keepalive => 1,
            header    => $header,
            ignoreredirects => 1,
            doTrigger => 1,
            callback  => \&ErrorHandling
        }
    );

    Log3 $name, 4,
    "HailoLibero ($name) - Send with URL: $hash->{URL}$uri, HEADER: secret!, DATA: secret!, METHOD: $method";

    # Log3($name, 3,
    #     "HailoLibero ($name) - Send with URL: $hash->{URL}$uri, HEADER: $header, DATA: $payload, METHOD: $method");

    return;
}

sub ErrorHandling {
    my $param = shift;
    my $err   = shift;
    my $data  = shift;

    my $hash  = $param->{hash};
    my $name  = $hash->{NAME};
    my $dhash = $hash;

    Log3 $name, 4, "HailoLibero ($name) - ErrorHandling ".$data;
    if ( ReadingsVal( $name, 'cookie', 'none' ) eq 'none' ){

      my ($logintry) = ($data =~ /(Invalid.+)<p/g);
        readingsSingleUpdate( $hash, 'state', $logintry, 1 );

      my (@cookies) = ($param->{httpheader} =~ /Set-Cookie: (.*)\s/g);
      foreach my $cookie (@cookies){
        readingsSingleUpdate( $hash, 'state', 'connected', 1 );
        readingsSingleUpdate( $hash, 'cookie', $cookie, 1 );
        Log3 $name, 4, 'eat cookie: '.$cookie;
      }
    } else {
      # got cookie ... do something
      #ledo pwro disto delayo sldrlbl
      if ($data eq 'OK'){
        getSettings($hash);
      }elsif (length($data) > 3 ){
        my ($deviceid) = ($data =~ /Libero30_[[:xdigit:]]{6}/g);
        my ($swversion) = ($data =~ /SW Version:(\d.\d.\d)/g);
        my ($wifi_ssid) = ($data =~ /STA-SSID:(\w+)/g);
        my ($wifi_ip) = ($data =~ /STA-IP:(\w+.\w+.\w+.\w+)/g);
        my ($wifi_netmask) = ($data =~ /STA-Subnet Mask:(\w+.\w+.\w+.\w+)/g);
        my ($led_brightness) = ($data =~ /led'>(\d*)/g); 
        my ($eject_power) = ($data =~ /pwr'>(\d*)/g);
        my ($detection_area) = ($data =~ /dist'>(\d*)/g);
        my ($detection_sensitivity) = ($data =~ /delay'>(\d*)/g);
        my ($eject_delay) = ($data =~ /pdelay'>(\d*)/g);
        readingsBeginUpdate($hash);
        readingsBulkUpdateIfChanged($hash, 'device', $deviceid);
        readingsBulkUpdateIfChanged($hash, 'firmware', $swversion);
        readingsBulkUpdateIfChanged($hash, 'wifi-ssid', $wifi_ssid);
        readingsBulkUpdateIfChanged($hash, 'wifi-ip', $wifi_ip);
        readingsBulkUpdateIfChanged($hash, 'wifi-subnet', $wifi_netmask);
      
        readingsBulkUpdateIfChanged($hash, 'led_brightness', $led_brightness);
        readingsBulkUpdateIfChanged($hash, 'eject_power', $eject_power);
        readingsBulkUpdateIfChanged($hash, 'detection_area', $detection_area);
        readingsBulkUpdateIfChanged($hash, 'detection_sensitivity', $detection_sensitivity);
        readingsBulkUpdateIfChanged($hash, 'eject_delay', $eject_delay);
        readingsEndUpdate( $hash, 1 );
      }
    }
    
    return;
}

sub createHttpValueStrings {
    my ( $hash, $payload ) = @_;
    my $name = $hash->{NAME};

    my $header     = "Content-Type: application/x-www-form-urlencoded";
    $header .= "\r\nCookie: ".ReadingsVal( $name, 'cookie', 'none' ) 
      if  (ReadingsVal( $name, 'cookie', 'none' ) ne 'none');
    my $uri        = '';
    my $method     = 'POST';
    $payload = '' if (!defined ($payload));

    if ( ReadingsVal( $name, 'cookie', 'none' ) eq 'none' ){
      $uri .= '/login';
      readingsSingleUpdate( $hash, 'state', 'fetch cookie', 1 );
    } elsif ( $payload ne 'open'
              && $payload ne ''){
      $uri .= '/settings';
    } elsif ( $payload eq 'open'){
      $method = 'GET';
      $uri .= '/push';
    }else {
      $method = 'GET';
      $uri .= '/admin';
    }
    return ( $payload, $header, $uri, $method );
}

####################################
####################################
#### my little helpers Sub's #######

sub ReadPassword {
    my $hash = shift;
    my $name = shift;

    my $password;

    Log3 $name, 4, "Hail ($name) - Read password from file";
    $password = $hash->{helper}->{passObj}->getReadPassword($name);

    if ( defined($password) ) {
        return $password;
    } else {
        Log3 $name, 3, "Hail ($name) - No password in file";
        return undef;
    }

    return;
}

sub Rename {
    my $new = shift;
    my $old = shift;

    my $hash = $defs{$new};

    $hash->{helper}->{passObj}->getRename($new,$old);
    return;
}


1;
